# Copyright (c) 2010-2011 Outpost Embedded, LLC
# This file is part of the pydbusclone software package.
#
# pydbusclone is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 2 as published by the
# Free Software Foundation.
#
# A copy of the license has been included in the COPYING file.

import dbus

from dbusclone.base import (
  DBusObject,
  DBusObjectWithDBusProperties,
  DBusObjectSet,
)
from dbusclone.util import CallbackList


class UDisks(DBusObject):
    devices = None

    device_changed_callbacks = None

    def __init__(self, **kwargs):
        super(UDisks, self).__init__(
          bus_name = 'org.freedesktop.UDisks',
          object_path = '/org/freedesktop/UDisks',
          **kwargs
        )

        self.devices = DBusObjectSet(
          bus = self.bus,
          object_cls = UDisksDevice,
          ready_callback = self._handle_devices_ready,
          default_error_handler = self.default_error_handler,
        )

        self.device_changed_callbacks = CallbackList()

        self.connect_to_signal('DeviceAdded', self.devices.add_object)
        self.connect_to_signal('DeviceRemoved', self.devices.remove_object)
        self.connect_to_signal('DeviceChanged', self._handle_device_changed)
        self.call(
          'EnumerateDevices',
          reply_handler = self.devices.add_initial_objects,
        )

    def get_proxy_object(self):
        obj = super(UDisks, self).get_proxy_object()
        return dbus.Interface(obj, 'org.freedesktop.UDisks')

    def is_ready(self):
        if not super(UDisks, self).is_ready():
            return False
        return ((self.devices is not None) and self.devices.is_ready())

    def _handle_devices_ready(self, devices):
        self.maybe_ready()

    def _handle_device_changed(self, device_path):
        try:
            device = self.devices.get_object(device_path)
        except KeyError:
            # FIXME: Under what circumstances would this happen?
            return

        self.device_changed_callbacks.call(device)

        # FIXME: Do we really need to do this?
        device.properties.refresh()


class UDisksDevice(DBusObjectWithDBusProperties):
    def __init__(self, **kwargs):
        super(UDisksDevice, self).__init__(
          bus_name = 'org.freedesktop.UDisks',
          properties_iface = 'org.freedesktop.UDisks.Device',
          **kwargs
        )

    def get_proxy_object(self):
        obj = super(UDisksDevice, self).get_proxy_object()
        return dbus.Interface(obj, 'org.freedesktop.UDisks.Device')
